//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//

#ifndef VKUserEventInformation_h
#define VKUserEventInformation_h 1

#include "VKSiPMScintInfo.hh"

#include "G4VUserEventInformation.hh"
#include "G4ThreeVector.hh"
#include "globals.hh"

#include <vector>

class VKUserEventInformation : public G4VUserEventInformation
{
  public:

    VKUserEventInformation();
    virtual ~VKUserEventInformation();

    inline virtual void Print() const{};

    void IncPhotonCount_Scint(size_t id);
    void IncPhotonCount_Ceren(size_t id);
    void IncEDep(size_t id, G4double dep);
    void IncAbsorption(size_t id);
    void IncBoundaryAbsorption(size_t id);
    void IncHitCount(size_t id, G4int i=1);

    void SetEWeightPos(size_t id, const G4ThreeVector& p);
    void SetReconPos(size_t id, const G4ThreeVector& p);
    void SetPosMax(size_t id, const G4ThreeVector& p, G4double edep);

    G4int GetPhotonCount_Scint(size_t id);
    G4int GetPhotonCount_Ceren(size_t id);
    G4int GetHitCount(size_t id);
    G4double GetEDep(size_t id);
    G4int GetAbsorptionCount(size_t id);
    G4int GetBoundaryAbsorptionCount(size_t id);

    G4ThreeVector GetEWeightPos(size_t id);
    G4ThreeVector GetReconPos(size_t id);
    G4ThreeVector GetPosMax(size_t id);
    G4double GetEDepMax(size_t id);

    // Gets the total photon count produced
    G4int GetPhotonCount(size_t id);
    G4int GetPhotonCount();

    void IncSiPMsAboveThreshold(size_t id);
    G4int GetSiPMsAboveThreshold(size_t id);


    void SetConvPos(const G4ThreeVector& p)  {fConvPos=p; fConvPosSet=true;}
    G4ThreeVector GetConvPos()               {return fConvPos;}
    G4double IsConvPosSet()                  {return fConvPosSet;}

  private:  
    void CheckCapacity(size_t id) const;
    void EnsureCapacity(size_t id);

    G4ThreeVector fConvPos;  // true (initial) conversion position
    G4bool fConvPosSet;
    
    std::vector<VKSiPMScintInfo*> info;

};

#endif
