//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//

#include "VKStackingAction.hh"
#include "VKUserEventInformation.hh"
#include "VKSteppingAction.hh"

#include "G4ios.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleTypes.hh"
#include "G4Track.hh"
#include "G4RunManager.hh"
#include "G4Event.hh"
#include "G4EventManager.hh"


VKStackingAction::VKStackingAction() 
{
    // do nothing
}


VKStackingAction::~VKStackingAction() 
{
    // do nothing
}


G4ClassificationOfNewTrack VKStackingAction::ClassifyNewTrack(const G4Track * aTrack)
{
 
    VKUserEventInformation* eventInfo = 
        (VKUserEventInformation*)G4EventManager::GetEventManager()
            -> GetConstCurrentEvent() -> GetUserInformation();
 
    // Count which process generated the optical photons
    if (aTrack->GetDefinition() == G4OpticalPhoton::OpticalPhotonDefinition()) // particle is optical photon
    {
        if (aTrack->GetParentID() > 0) // particle is secondary
        {
            G4int scintId = 0;
            auto name = aTrack->GetVolume()->GetName();
            if (name == "signal") scintId = 0;
            else if (name == "veto") scintId = 1;
                    
            if (aTrack->GetCreatorProcess()->GetProcessName() == "Scintillation")
                eventInfo->IncPhotonCount_Scint(scintId);
            else if (aTrack->GetCreatorProcess()->GetProcessName() == "Cerenkov")
                eventInfo->IncPhotonCount_Ceren(scintId);
        }
    }

    return fUrgent;
}


void VKStackingAction::NewStage() 
{
    // do nothing
}


void VKStackingAction::PrepareNewEvent() 
{
    // do nothing
}
