//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//

#include "VKUserEventInformation.hh"


VKUserEventInformation::VKUserEventInformation()
 : fConvPos(0.),
   fConvPosSet(false)
{
    // do nothing
}


VKUserEventInformation::~VKUserEventInformation() 
{
    for (auto& i : info) delete i;
}


void VKUserEventInformation::IncPhotonCount_Scint(size_t id) 
{
    EnsureCapacity(id);
    info[id]->IncPhotonCount_Scint();
}

void VKUserEventInformation::IncPhotonCount_Ceren(size_t id) 
{
    EnsureCapacity(id);
    info[id]->IncPhotonCount_Ceren();
}

void VKUserEventInformation::IncEDep(size_t id, G4double dep) 
{
    EnsureCapacity(id);
    info[id]->IncEDep(dep);
}

void VKUserEventInformation::IncAbsorption(size_t id) 
{
    EnsureCapacity(id);
    info[id]->IncAbsorption();
}

void VKUserEventInformation::IncBoundaryAbsorption(size_t id) 
{
    EnsureCapacity(id);
    info[id]->IncBoundaryAbsorption();
}

void VKUserEventInformation::IncHitCount(size_t id, G4int i) 
{
    EnsureCapacity(id);
    info[id]->IncHitCount(i);
}

void VKUserEventInformation::SetEWeightPos(size_t id, const G4ThreeVector& p) 
{
    EnsureCapacity(id);
    info[id]->SetEWeightPos(p);
}

void VKUserEventInformation::SetReconPos(size_t id, const G4ThreeVector& p) 
{
    EnsureCapacity(id);
    info[id]->SetReconPos(p);
}

void VKUserEventInformation::SetPosMax(size_t id, const G4ThreeVector& p, G4double edep) 
{
    EnsureCapacity(id);
    info[id]->SetPosMax(p, edep);
}

void VKUserEventInformation::IncSiPMsAboveThreshold(size_t id)
{
    EnsureCapacity(id);
    info[id]->IncSiPMsAboveThreshold();
}

G4int VKUserEventInformation::GetPhotonCount_Scint(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetPhotonCount_Scint();
}

G4int VKUserEventInformation::GetPhotonCount_Ceren(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetPhotonCount_Ceren();
}

G4int VKUserEventInformation::GetHitCount(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetHitCount();
}

G4double VKUserEventInformation::GetEDep(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetEDep();
}

G4int VKUserEventInformation::GetAbsorptionCount(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetAbsorptionCount();
}

G4int VKUserEventInformation::GetBoundaryAbsorptionCount(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetBoundaryAbsorptionCount();
}

G4ThreeVector VKUserEventInformation::GetEWeightPos(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetEWeightPos();
}

G4ThreeVector VKUserEventInformation::GetReconPos(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetReconPos();
}

G4ThreeVector VKUserEventInformation::GetPosMax(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetPosMax();
}

G4double VKUserEventInformation::GetEDepMax(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetEDepMax();
}

G4int VKUserEventInformation::GetPhotonCount(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetPhotonCount();
}

G4int VKUserEventInformation::GetPhotonCount()
{
    G4int photons = 0;
    for (auto& i : info) photons += i->GetPhotonCount();
    return photons;
}

G4int VKUserEventInformation::GetSiPMsAboveThreshold(size_t id)
{
    EnsureCapacity(id);
    return info[id]->GetSiPMsAboveThreshold();
}
    
void VKUserEventInformation::EnsureCapacity(size_t id)
{
    if (id >= info.size()) info.emplace_back(new VKSiPMScintInfo());
}

void VKUserEventInformation::CheckCapacity(size_t id) const
{
    if (id >= info.size())
    {
        G4ExceptionDescription ed;
        ed << "VKUserEventInformation::CheckCapacity(size_t): "
        << "Attempt to access information for non-existing SiPM-Scintillator unit (id: " << id << ")"
        << G4endl;
        G4Exception("VKUserEventInformation::CheckCapacity(size_t):", "VK",
        FatalException, ed, "Something is wrong with number of SiPM-Scintillator unit");    
    }
}
