//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "globals.hh"

#include "VKVirtualDetector.hh"
#include "VKUtil.hh"

#include "G4LogicalSkinSurface.hh"
#include "G4LogicalBorderSurface.hh"
#include "G4SystemOfUnits.hh"
#include "G4Cons.hh"


VKVirtualDetector::VKVirtualDetector(G4RotationMatrix *pRot,
                             const G4ThreeVector &tlate,
                             G4LogicalVolume *pMother_log,
                             G4bool pMany,
                             G4int pCopyNo,
                             const VKDetectorConstruction* pDetConstr,
                             G4String name,
                             G4double thickness,
                             G4double rmin,
                             G4double rmax)

 // Pass info to the G4PVPlacement constructor
 : G4PVPlacement(pRot, 
                 tlate,
                 CreateTemporaryLogicalVolume(), // <-- see VKUtil.cpp
                 name, 
                 pMother_log, 
                 pMany, 
                 pCopyNo),
   fDetConstr(pDetConstr),              
   fMother_log(pMother_log),
   fThickness(thickness),
   fInnerRadius(rmin),
   fOuterRadius(rmax)
{

    // default dimensions    
    SetDefaults();

    // List of materials
    fMaterials = VKMaterials::GetInstance();

    // Get materials
    G4Material* labVacuum = fMaterials -> GetMaterial("LABVACUUM");
    assert(labVacuum != nullptr);
    
    // solid
    fPlane_sol = 
        new G4Tubs("plane_sol", fInnerRadius, fOuterRadius, fThickness/2, 0.*deg, 360.*deg);

    // logical
    fPlane_log = 
        new G4LogicalVolume(fPlane_sol, labVacuum, "plane_log");

    // Set the logical volume
    this -> SetLogicalVolume(fPlane_log);

    // visualization attributes
    VisAttributes();    
    
    // Move the physical volume
    this -> SetTranslation(tlate);
    
    // Check for overlaps
    if (fCheckOverlaps) this -> CheckOverlaps();
}


void VKVirtualDetector::SetDefaults(){
    fCheckOverlaps = fDetConstr -> GetCheckOverlaps();
}


void VKVirtualDetector::VisAttributes() 
{
    // do nothing
}
